@@ -38,6 +38,7 @@
 #define SMB_COM_WRITE_ANDX            0x2F
 #define SMB_COM_TRANSACTION2	      0x32
 #define SMB_COM_TRANSACTION2_SECONDARY 0x33
+#define SMB_COM_FIND_CLOSE2           0x34
 #define SMB_COM_TREE_DISCONNECT       0x71
 #define SMB_COM_NEGOTIATE             0x72
 #define SMB_COM_SESSION_SETUP_ANDX    0x73
@@ -79,7 +80,7 @@
 /*
  * Starting value for maximum SMB size negotiation
  */
-#define CIFS_MAX_MSGSIZE (4*1024)
+#define CIFS_MAX_MSGSIZE (4*4096)
 
 /*
  * Size of encrypted user password in bytes
@@ -188,13 +189,11 @@
 					 /* file_execute, file_read_attributes */
 					 /* write_dac, and delete.           */
 
-
 /*
  * Invalid readdir handle
  */
 #define CIFS_NO_HANDLE        0xFFFF
 
-
 /* IPC$ in ASCII */
 #define CIFS_IPC_RESOURCE "\x49\x50\x43\x24"
 
@@ -205,7 +204,6 @@
 #define UNICODE_NULL "\x00\x00"
 #define ASCII_NULL 0x00
 
-
 /*
  * Server type values (returned on EnumServer API
  */
@@ -237,6 +235,8 @@
 #define ATTR_NOT_CONTENT_INDEXED 0x2000
 #define ATTR_ENCRYPTED  0x4000
 #define ATTR_POSIX_SEMANTICS 0x01000000
+#define ATTR_BACKUP_SEMANTICS 0x02000000
+#define ATTR_DELETE_ON_CLOSE 0x04000000
 #define ATTR_SEQUENTIAL_SCAN 0x08000000
 #define ATTR_RANDOM_ACCESS   0x10000000
 #define ATTR_NO_BUFFERING    0x20000000
@@ -258,8 +258,12 @@
 #define FILE_OVERWRITE_IF 0x00000005
 
 /* CreateOptions */
-#define CREATE_NOT_FILE   0x00000001	/* if set, indicates must not be file */
-#define CREATE_NOT_DIR    0x00000040	/* if set, indicates must not be directory */
+#define CREATE_NOT_FILE		0x00000001	/* if set must not be file */
+#define CREATE_WRITE_THROUGH	0x00000002
+#define CREATE_NOT_DIR		0x00000040	/* if set must not be directory */
+#define CREATE_RANDOM_ACCESS	0x00000800
+#define CREATE_DELETE_ON_CLOSE	0x00001000
+#define OPEN_REPARSE_POINT	0x00200000
 
 /* ImpersonationLevel flags */
 #define SECURITY_ANONYMOUS      0
@@ -271,7 +275,6 @@
 #define SECURITY_CONTEXT_TRACKING 0x01
 #define SECURITY_EFFECTIVE_ONLY   0x02
 
-
 /*
  * Default PID value, used in all SMBs where the PID is not important
  */
@@ -289,7 +292,7 @@
 
 #pragma pack(1)
 
-typedef struct smb_hdr {
+struct smb_hdr {
 	__u32 smb_buf_length;	/* big endian on wire *//* BB length is only two or three bytes - with one or two byte type preceding it but that is always zero - we could mask the type byte off just in case BB */
 	__u8 Protocol[4];
 	__u8 Command;
@@ -311,7 +314,7 @@ typedef struct smb_hdr {
 	__u16 Uid;
 	__u16 Mid;
 	__u8 WordCount;
-} SMB_HDR;
+};
 /* given a pointer to an smb_hdr retrieve the value of byte count */
 #define BCC(smb_var) ( *(__u16 *)((char *)smb_var + sizeof(struct smb_hdr) + (2* smb_var->WordCount) ) )
 
@@ -341,25 +344,32 @@ typedef struct smb_hdr {
  */
 #define MAX_PATHCONF 256
 
-
 /*
  *  SMB frame definitions  (following must be packed structs)
  *  See the SNIA CIFS Specification for details.
  *
- *  The Naming convention for the unions is the
- *  lower case version of the smb command code name and this
- *  union is typedef to the uppercase version of the same name
- *  with the prefix SMB_ removed for brevity.
+ *  The Naming convention is the lower case version of the
+ *  smb command code name for the struct and this is typedef to the
+ *  uppercase version of the same name with the prefix SMB_ removed 
+ *  for brevity.  Although typedefs are not commonly used for 
+ *  structure definitions in the Linux kernel, their use in the
+ *  CIFS standards document, which this code is based on, may
+ *  make this one of the cases where typedefs for structures make
+ *  sense to improve readability for readers of the standards doc.
+ *  Typedefs can always be removed later if they are too distracting
+ *  and they are only used for the CIFSs PDUs themselves, not
+ *  internal cifs vfs structures
  *  
  */
+
 typedef struct negotiate_req {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;
 	unsigned char DialectsArray[1];
 } NEGOTIATE_REQ;
 
 typedef struct negotiate_rsp {
-	SMB_HDR hdr;		/* wct = 17 */
+	struct smb_hdr hdr;	/* wct = 17 */
 	__u16 DialectIndex;
 	__u8 SecurityMode;
 	__u16 MaxMpxCount;
@@ -412,10 +422,9 @@ typedef struct negotiate_rsp {
 #define CAP_COMPRESSED_DATA    0x40000000
 #define CAP_EXTENDED_SECURITY  0x80000000
 
-
 typedef union smb_com_session_setup_andx {
 	struct {		/* request format */
-		SMB_HDR hdr;	/* wct = 12 */
+		struct smb_hdr hdr;	/* wct = 12 */
 		__u8 AndXCommand;
 		__u8 AndXReserved;
 		__u16 AndXOffset;
@@ -433,7 +442,7 @@ typedef union smb_com_session_setup_andx {
 	} req;			/* NTLM request format (with extended security */
 
 	struct {		/* request format */
-		SMB_HDR hdr;	/* wct = 13 */
+		struct smb_hdr hdr;	/* wct = 13 */
 		__u8 AndXCommand;
 		__u8 AndXReserved;
 		__u16 AndXOffset;
@@ -455,7 +464,7 @@ typedef union smb_com_session_setup_andx {
 	} req_no_secext;	/* NTLM request format (without extended security */
 
 	struct {		/* default (NTLM) response format */
-		SMB_HDR hdr;	/* wct = 4 */
+		struct smb_hdr hdr;	/* wct = 4 */
 		__u8 AndXCommand;
 		__u8 AndXReserved;
 		__u16 AndXOffset;
@@ -469,7 +478,7 @@ typedef union smb_com_session_setup_andx {
 	} resp;			/* NTLM response format (with or without extended security */
 
 	struct {		/* request format */
-		SMB_HDR hdr;	/* wct = 10 */
+		struct smb_hdr hdr;	/* wct = 10 */
 		__u8 AndXCommand;
 		__u8 AndXReserved;
 		__u16 AndXOffset;
@@ -488,7 +497,7 @@ typedef union smb_com_session_setup_andx {
 	} old_req;		/* pre-NTLM (LANMAN2.1) request format */
 
 	struct {		/* default (NTLM) response format */
-		SMB_HDR hdr;	/* wct = 3 */
+		struct smb_hdr hdr;	/* wct = 3 */
 		__u8 AndXCommand;
 		__u8 AndXReserved;
 		__u16 AndXOffset;
@@ -516,7 +525,7 @@ typedef union smb_com_session_setup_andx {
 #define GUEST_LOGIN 1
 
 typedef struct smb_com_tconx_req {
-	SMB_HDR hdr;		/* wct = 4 */
+	struct smb_hdr hdr;	/* wct = 4 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -529,7 +538,7 @@ typedef struct smb_com_tconx_req {
 } TCONX_REQ;
 
 typedef struct smb_com_tconx_rsp {
-	SMB_HDR hdr;		/* wct = 3 *//* note that Win2000 has sent wct=7 in some cases on responses. Four unspecified words followed OptionalSupport */
+	struct smb_hdr hdr;	/* wct = 3 *//* note that Win2000 has sent wct=7 in some cases on responses. Four unspecified words followed OptionalSupport */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -546,10 +555,9 @@ typedef struct smb_com_tconx_rsp {
 #define SMB_SUPPORT_SEARCH_BITS 0x0001	/* must have bits (exclusive searches suppt. */
 #define SMB_SHARE_IS_IN_DFS     0x0002
 
-
 typedef struct smb_com_logoff_andx_req {
 
-	SMB_HDR hdr;		/* wct = 2 */
+	struct smb_hdr hdr;	/* wct = 2 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -557,7 +565,7 @@ typedef struct smb_com_logoff_andx_req {
 } LOGOFF_ANDX_REQ;
 
 typedef struct smb_com_logoff_andx_rsp {
-	SMB_HDR hdr;		/* wct = 2 */
+	struct smb_hdr hdr;	/* wct = 2 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -566,34 +574,40 @@ typedef struct smb_com_logoff_andx_rsp {
 
 typedef union smb_com_tree_disconnect {	/* as an altetnative can use flag on tree_connect PDU to effect disconnect *//* probably the simplest SMB PDU */
 	struct {
-		SMB_HDR hdr;	/* wct = 0 */
+		struct smb_hdr hdr;	/* wct = 0 */
 		__u16 ByteCount;	/* bcc = 0 */
 	} req;
 	struct {
-		SMB_HDR hdr;	/* wct = 0 */
+		struct smb_hdr hdr;	/* wct = 0 */
 		__u16 ByteCount;	/* bcc = 0 */
 	} resp;
 } TREE_DISCONNECT;
 
 typedef struct smb_com_close_req {
-	SMB_HDR hdr;		/* wct = 3 */
-	__u16 FileID;		/* target file attributes */
+	struct smb_hdr hdr;	/* wct = 3 */
+	__u16 FileID;
 	__u32 LastWriteTime;	/* should be zero */
 	__u16 ByteCount;	/* 0 */
 } CLOSE_REQ;
 
 typedef struct smb_com_close_rsp {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;	/* bct = 0 */
 } CLOSE_RSP;
 
+typedef struct smb_com_findclose_req {
+	struct smb_hdr hdr; /* wct = 1 */
+	__u16 FileID;
+	__u16 ByteCount;    /* 0 */
+} FINDCLOSE_REQ;
+
 /* OpenFlags */
 #define REQ_OPLOCK         0x00000002
 #define REQ_BATCHOPLOCK    0x00000004
 #define REQ_OPENDIRONLY    0x00000008
 
 typedef struct smb_com_open_req {	/* also handles create */
-	SMB_HDR hdr;		/* wct = 24 */
+	struct smb_hdr hdr;	/* wct = 24 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -614,7 +628,7 @@ typedef struct smb_com_open_req {	/* also handles create */
 } OPEN_REQ;
 
 typedef struct smb_com_open_rsp {
-	SMB_HDR hdr;		/* wct = 34 BB */
+	struct smb_hdr hdr;	/* wct = 34 BB */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -634,9 +648,8 @@ typedef struct smb_com_open_rsp {
 	__u16 ByteCount;	/* bct = 0 */
 } OPEN_RSP;
 
-
 typedef struct smb_com_write_req {
-	SMB_HDR hdr;		/* wct = 14 */
+	struct smb_hdr hdr;	/* wct = 14 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -655,7 +668,7 @@ typedef struct smb_com_write_req {
 } WRITE_REQ;
 
 typedef struct smb_com_write_rsp {
-	SMB_HDR hdr;		/* wct = 6 */
+	struct smb_hdr hdr;	/* wct = 6 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -666,7 +679,7 @@ typedef struct smb_com_write_rsp {
 } WRITE_RSP;
 
 typedef struct smb_com_read_req {
-	SMB_HDR hdr;		/* wct = 12 */
+	struct smb_hdr hdr;	/* wct = 12 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -681,7 +694,7 @@ typedef struct smb_com_read_req {
 } READ_REQ;
 
 typedef struct smb_com_read_rsp {
-	SMB_HDR hdr;		/* wct = 12 */
+	struct smb_hdr hdr;	/* wct = 12 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -711,7 +724,7 @@ typedef struct locking_andx_range {
 #define LOCKING_ANDX_LARGE_FILES     0x10	/* always on for us */
 
 typedef struct smb_com_lock_req {
-	SMB_HDR hdr;		/* wct = 8 */
+	struct smb_hdr hdr;	/* wct = 8 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
@@ -726,16 +739,15 @@ typedef struct smb_com_lock_req {
 } LOCK_REQ;
 
 typedef struct smb_com_lock_rsp {
-	SMB_HDR hdr;		/* wct = 2 */
+	struct smb_hdr hdr;	/* wct = 2 */
 	__u8 AndXCommand;
 	__u8 AndXReserved;
 	__u16 AndXOffset;
 	__u16 ByteCount;
 } LOCK_RSP;
 
-
 typedef struct smb_com_rename_req {
-	SMB_HDR hdr;		/* wct = 1 */
+	struct smb_hdr hdr;	/* wct = 1 */
 	__u16 SearchAttributes;	/* target file attributes */
 	__u16 ByteCount;
 	__u8 BufferFormat;	/* 4 = ASCII or Unicode */
@@ -744,9 +756,12 @@ typedef struct smb_com_rename_req {
 	/* followed by NewFileName */
 } RENAME_REQ;
 
-#define CREATE_HARD_LINK 0x103
+#define CREATE_HARD_LINK		0x103
+#define MOVEFILE_COPY_ALLOWED		0x0002
+#define MOVEFILE_REPLACE_EXISTING	0x0001
+
 typedef struct smb_com_nt_rename_req {	/* A5 - also used for create hardlink */
-	SMB_HDR hdr;		/* wct = 4 */
+	struct smb_hdr hdr;	/* wct = 4 */
 	__u16 SearchAttributes;	/* target file attributes */
 	__u16 Flags;		/* spec says Information Level */
 	__u32 ClusterCount;
@@ -757,14 +772,13 @@ typedef struct smb_com_nt_rename_req {	/* A5 - also used for create hardlink */
 	/* followed by NewFileName */
 } NT_RENAME_REQ;
 
-
 typedef struct smb_com_rename_rsp {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;	/* bct = 0 */
 } RENAME_RSP;
 
 typedef struct smb_com_delete_file_req {
-	SMB_HDR hdr;		/* wct = 1 */
+	struct smb_hdr hdr;	/* wct = 1 */
 	__u16 SearchAttributes;
 	__u16 ByteCount;
 	__u8 BufferFormat;	/* 4 = ASCII */
@@ -772,36 +786,36 @@ typedef struct smb_com_delete_file_req {
 } DELETE_FILE_REQ;
 
 typedef struct smb_com_delete_file_rsp {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;	/* bct = 0 */
 } DELETE_FILE_RSP;
 
 typedef struct smb_com_delete_directory_req {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;
 	__u8 BufferFormat;	/* 4 = ASCII */
 	unsigned char DirName[1];
 } DELETE_DIRECTORY_REQ;
 
 typedef struct smb_com_delete_directory_rsp {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;	/* bct = 0 */
 } DELETE_DIRECTORY_RSP;
 
 typedef struct smb_com_create_directory_req {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;
 	__u8 BufferFormat;	/* 4 = ASCII */
 	unsigned char DirName[1];
 } CREATE_DIRECTORY_REQ;
 
 typedef struct smb_com_create_directory_rsp {
-	SMB_HDR hdr;		/* wct = 0 */
+	struct smb_hdr hdr;	/* wct = 0 */
 	__u16 ByteCount;	/* bct = 0 */
 } CREATE_DIRECTORY_RSP;
 
-typedef struct smb_com_nt_transaction_ioctl_req {
-	SMB_HDR hdr;		/* wct = 23 */
+typedef struct smb_com_transaction_ioctl_req {
+	struct smb_hdr hdr;	/* wct = 23 */
 	__u8 MaxSetupCount;
 	__u16 Reserved;
 	__u32 TotalParameterCount;
@@ -812,20 +826,20 @@ typedef struct smb_com_nt_transaction_ioctl_req {
 	__u32 ParameterOffset;
 	__u32 DataCount;
 	__u32 DataOffset;
-	__u8 SetupCount;	/* four setup words follow subcommand */
+	__u8 SetupCount; /* four setup words follow subcommand */
 	/* SNIA spec incorrectly included spurious pad here */
-	__u16 SubCommand;	/* 2 = IOCTL/FSCTL */
+	__u16 SubCommand;/* 2 = IOCTL/FSCTL */
 	__u32 FunctionCode;
 	__u16 Fid;
-	__u8 IsFSCTLFlag;	/* 1 = File System Control, 0 = device control (IOCTL)    */
-	__u8 IsRootFlag;	/* 1 = apply command to root of share (must be DFS share) */
+	__u8 IsFsctl;    /* 1 = File System Control, 0 = device control (IOCTL)*/
+	__u8 IsRootFlag; /* 1 = apply command to root of share (must be DFS share)*/
 	__u16 ByteCount;
 	__u8 Pad[3];
 	__u8 Data[1];
 } TRANSACT_IOCTL_REQ;
 
 typedef struct smb_com_transaction_ioctl_rsp {
-	SMB_HDR hdr;		/* wct = 19 */
+	struct smb_hdr hdr;	/* wct = 19 */
 	__u8 Reserved[3];
 	__u32 TotalParameterCount;
 	__u32 TotalDataCount;
@@ -841,11 +855,20 @@ typedef struct smb_com_transaction_ioctl_rsp {
 	__u8 Pad[3];
 } TRANSACT_IOCTL_RSP;
 
-
+struct reparse_data {
+	__u32	ReparseTag;
+	__u16	ReparseDataLength;
+	__u16	Reserved;
+	__u16	AltNameOffset;
+	__u16	AltNameLen;
+	__u16	TargetNameOffset;
+	__u16	TargetNameLen;
+	char	LinkNamesBuf[1];
+};
 
 typedef union smb_com_transaction2 {
 	struct {
-		SMB_HDR hdr;	/* wct = 14+ */
+		struct smb_hdr hdr;	/* wct = 14+ */
 		__u16 TotalParameterCount;
 		__u16 TotalDataCount;
 		__u16 MaxParameterCount;
@@ -865,7 +888,7 @@ typedef union smb_com_transaction2 {
 		__u16 ByteCount;	/* careful - setupcount is not always one */
 	} req;
 	struct {
-		SMB_HDR hdr;	/* wct = 0 */
+		struct smb_hdr hdr;	/* wct = 0 */
 		__u16 TotalParameterCount;
 		__u16 TotalDataCount;
 		__u16 Reserved;
@@ -898,7 +921,7 @@ typedef union smb_com_transaction2 {
 #define SMB_QUERY_FILE_UNIX_BASIC    0x200
 #define SMB_QUERY_FILE_UNIX_LINK     0x201
 
-#define SMB_SET_FILE_BASIC_INFO		0x101
+#define SMB_SET_FILE_BASIC_INFO			0x101
 #define SMB_SET_FILE_DISPOSITION_INFO	0x102
 #define SMB_SET_FILE_ALLOCATION_INFO	0x103
 #define SMB_SET_FILE_END_OF_FILE_INFO	0x104
@@ -909,17 +932,15 @@ typedef union smb_com_transaction2 {
 #define SMB_SET_FILE_ALLOCATION_INFO2	0x3fb
 #define SMB_SET_FILE_END_OF_FILE_INFO2	0x3fc
 
-
 /* Find File infolevels */
 #define SMB_FIND_FILE_DIRECTORY_INFO	  0x101
 #define SMB_FIND_FILE_FULL_DIRECTORY_INFO 0x102
-#define SMB_FIND_FILE_NAMES_INFO	  0x103
+#define SMB_FIND_FILE_NAMES_INFO		0x103
 #define SMB_FIND_FILE_BOTH_DIRECTORY_INFO 0x104
 #define SMB_FIND_FILE_UNIX                0x202
 
-
 typedef struct smb_com_transaction2_qpi_req {
-	SMB_HDR hdr;		/* wct = 14+ */
+	struct smb_hdr hdr;	/* wct = 14+ */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 MaxParameterCount;
@@ -944,7 +965,7 @@ typedef struct smb_com_transaction2_qpi_req {
 } TRANSACTION2_QPI_REQ;
 
 typedef struct smb_com_transaction2_qpi_rsp {
-	SMB_HDR hdr;		/* wct = 10 + SetupCount */
+	struct smb_hdr hdr;	/* wct = 10 + SetupCount */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 Reserved;
@@ -961,8 +982,7 @@ typedef struct smb_com_transaction2_qpi_rsp {
 } TRANSACTION2_QPI_RSP;
 
 typedef struct smb_com_transaction2_spi_req {
-
-	SMB_HDR hdr;		/* wct = 15 */
+	struct smb_hdr hdr;	/* wct = 15 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 MaxParameterCount;
@@ -988,7 +1008,7 @@ typedef struct smb_com_transaction2_spi_req {
 } TRANSACTION2_SPI_REQ;
 
 typedef struct smb_com_transaction2_spi_rsp {
-	SMB_HDR hdr;		/* wct = 10 + SetupCount */
+	struct smb_hdr hdr;	/* wct = 10 + SetupCount */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 Reserved;
@@ -1004,6 +1024,49 @@ typedef struct smb_com_transaction2_spi_rsp {
 	__u16 Reserved2;	/* parameter word reserved - present for infolevels > 100 */
 } TRANSACTION2_SPI_RSP;
 
+struct smb_com_transaction2_sfi_req {
+	struct smb_hdr hdr;	/* wct = 15 */
+	__u16 TotalParameterCount;
+	__u16 TotalDataCount;
+	__u16 MaxParameterCount;
+	__u16 MaxDataCount;
+	__u8 MaxSetupCount;
+	__u8 Reserved;
+	__u16 Flags;
+	__u32 Timeout;
+	__u16 Reserved2;
+	__u16 ParameterCount;
+	__u16 ParameterOffset;
+	__u16 DataCount;
+	__u16 DataOffset;
+	__u8 SetupCount;
+	__u8 Reserved3;
+	__u16 SubCommand;	/* one setup word */
+	__u16 ByteCount;
+	__u8 Pad;
+	__u16 Pad1;
+    __u16 Fid;
+	__u16 InformationLevel;
+	__u16 Reserved4;	
+};
+
+struct smb_com_transaction2_sfi_rsp {
+	struct smb_hdr hdr;	/* wct = 10 + SetupCount */
+	__u16 TotalParameterCount;
+	__u16 TotalDataCount;
+	__u16 Reserved;
+	__u16 ParameterCount;
+	__u16 ParameterOffset;
+	__u16 ParameterDisplacement;
+	__u16 DataCount;
+	__u16 DataOffset;
+	__u16 DataDisplacement;
+	__u8 SetupCount;
+	__u8 Reserved1;		/* should be zero setup words following */
+	__u16 ByteCount;
+	__u16 Reserved2;	/* parameter word reserved - present for infolevels > 100 */
+};
+
 
 /*
  * Flags on T2 FINDFIRST and FINDNEXT 
@@ -1019,9 +1082,8 @@ typedef struct smb_com_transaction2_spi_rsp {
  */
 #define CIFS_SMB_RESUME_KEY_SIZE 4
 
-
 typedef struct smb_com_transaction2_ffirst_req {
-	SMB_HDR hdr;		/* wct = 15 */
+	struct smb_hdr hdr;	/* wct = 15 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 MaxParameterCount;
@@ -1049,7 +1111,7 @@ typedef struct smb_com_transaction2_ffirst_req {
 } TRANSACTION2_FFIRST_REQ;
 
 typedef struct smb_com_transaction2_ffirst_rsp {
-	SMB_HDR hdr;		/* wct = 10 */
+	struct smb_hdr hdr;	/* wct = 10 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 Reserved;
@@ -1073,7 +1135,7 @@ typedef struct smb_com_transaction2_ffirst_rsp_parms {
 } T2_FFIRST_RSP_PARMS;
 
 typedef struct smb_com_transaction2_fnext_req {
-	SMB_HDR hdr;		/* wct = 15 */
+	struct smb_hdr hdr;	/* wct = 15 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 MaxParameterCount;
@@ -1097,11 +1159,11 @@ typedef struct smb_com_transaction2_fnext_req {
 	__u16 InformationLevel;
 	__u32 ResumeKey;
 	__u16 SearchFlags;
-	char ResumeFileName[1];	/* will be null string actually since we set bit 3 - resume from previous ending place */
+	char ResumeFileName[1];
 } TRANSACTION2_FNEXT_REQ;
 
 typedef struct smb_com_transaction2_fnext_rsp {
-	SMB_HDR hdr;		/* wct = 10 */
+	struct smb_hdr hdr;	/* wct = 10 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 Reserved;
@@ -1132,7 +1194,7 @@ typedef struct smb_com_transaction2_fnext_rsp_parms {
 #define SMB_QUERY_CIFS_UNIX_INFO    0x200
 
 typedef struct smb_com_transaction2_qfsi_req {
-	SMB_HDR hdr;		/* wct = 14+ */
+	struct smb_hdr hdr;	/* wct = 14+ */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 MaxParameterCount;
@@ -1155,7 +1217,7 @@ typedef struct smb_com_transaction2_qfsi_req {
 } TRANSACTION2_QFSI_REQ;
 
 typedef struct smb_com_transaction_qfsi_rsp {
-	SMB_HDR hdr;		/* wct = 10 + SetupCount */
+	struct smb_hdr hdr;	/* wct = 10 + SetupCount */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 Reserved;
@@ -1172,7 +1234,7 @@ typedef struct smb_com_transaction_qfsi_rsp {
 } TRANSACTION2_QFSI_RSP;
 
 typedef struct smb_com_transaction2_get_dfs_refer_req {
-	SMB_HDR hdr;		/* wct = 15 */
+	struct smb_hdr hdr;	/* wct = 15 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 MaxParameterCount;
@@ -1207,7 +1269,7 @@ typedef struct dfs_referral_level_3 {
 } REFERRAL3;
 
 typedef struct smb_com_transaction_get_dfs_refer_rsp {
-	SMB_HDR hdr;		/* wct = 10 */
+	struct smb_hdr hdr;	/* wct = 10 */
 	__u16 TotalParameterCount;
 	__u16 TotalDataCount;
 	__u16 Reserved;
@@ -1266,7 +1328,7 @@ typedef struct smb_com_transaction_get_dfs_refer_rsp {
 
 /*
  ************************************************************************
- * All typedefs and structs for everything above the SMB PDUs themselves
+ * All structs for everything above the SMB PDUs themselves
  * (such as the T2 level specific data) go here                  
  ************************************************************************
  */
@@ -1275,38 +1337,38 @@ typedef struct smb_com_transaction_get_dfs_refer_rsp {
  * Information on a server
  */
 
-typedef struct {
+struct serverInfo {
 	char name[16];
 	unsigned char versionMajor;
 	unsigned char versionMinor;
 	unsigned long type;
 	unsigned int commentOffset;
-} serverInfo;
+};
 
 /*
  * The following structure is the format of the data returned on a NetShareEnum
  * with level "90" (x5A)
  */
 
-typedef struct {
+struct shareInfo {
 	char shareName[13];
 	char pad;
 	unsigned short type;
 	unsigned int commentOffset;
-} shareInfo;
+};
 
-typedef struct {
+struct aliasInfo {
 	char aliasName[9];
 	char pad;
 	unsigned int commentOffset;
 	unsigned char type[2];
-} aliasInfo;
+};
 
-typedef struct {
+struct aliasInfo92 {
 	int aliasNameOffset;
 	int serverNameOffset;
 	int shareNameOffset;
-} aliasInfo92;
+};
 
 typedef struct {
 	__u64 TotalAllocationUnits;
@@ -1321,7 +1383,6 @@ typedef struct {
 	__u64 Capability;
 } FILE_SYSTEM_UNIX_INFO;	/* Unix extensions info, level 0x200 */
 
-
 /* DeviceType Flags */
 #define FILE_DEVICE_CD_ROM              0x00000002
 #define FILE_DEVICE_CD_ROM_FILE_SYSTEM  0x00000003
@@ -1398,7 +1459,6 @@ typedef struct {
 	char LinkDest[1];
 } FILE_UNIX_LINK_INFO;		/* level 513 QPathInfo */
 
-
 /* defines for enumerating possible values of the Unix type field below */
 #define UNIX_FILE      0
 #define UNIX_DIR       1
@@ -1436,13 +1496,13 @@ typedef struct {
 	__u32 Pad;
 } FILE_BASIC_INFO;		/* size info, level 0x101 */
 
-typedef struct {
+struct file_allocation_info {
 	__u64 AllocationSize;
-} FILE_ALLOCATION_INFO;		/* size info, level 0x103 */
+};		/* size info, level 0x103 */
 
-typedef struct {
+struct file_end_of_file_info {
 	__u64 FileSize;		/* offset to end of file */
-} FILE_END_OF_FILE_INFO;	/* size info, level 0x104 */
+};	/* size info, level 0x104 */
 
 typedef struct {
 	__u32 NextEntryOffset;
@@ -1456,38 +1516,38 @@ typedef struct {
 	__u32 ExtFileAttributes;
 	__u32 FileNameLength;
 	char FileName[1];
-} FILE_DIRECTORY_INFO;		/* level 257 FF response data area */
+} FILE_DIRECTORY_INFO;   /* level 257 FF response data area */
 
-typedef struct {
+struct gea {
 	unsigned char cbName;
 	char szName[1];
-} GEA;
+};
 
-typedef struct {
+struct gealist {
 	unsigned long cbList;
-	GEA list[1];
-} GEALIST;
+	struct gea list[1];
+};
 
-typedef struct {
+struct fea {
 	unsigned char fEA;
 	unsigned char cbName;
 	unsigned short cbValue;
 	char szName[1];
-} FEA;
+};
 /* flags for _FEA.fEA */
 #define FEA_NEEDEA         0x80	/* need EA bit */
 
-typedef struct {
+struct fealist {
 	unsigned long cbList;
-	FEA list[1];
-} FEALIST;
+	struct fea list[1];
+};
 
 /* used to hold an arbitrary blob of data */
-typedef struct data_blob {
+struct data_blob {
 	__u8 *data;
 	size_t length;
 	void (*free) (struct data_blob * data_blob);
-} DATA_BLOB;
+};
 
 #pragma pack()			/* resume default structure packing */
 
